#!/usr/bin/perl

######################################
# Soldat Server Statistics Generator #
# Version 0.20                       #
# (c) 2003 CREEQ                     #
######################################

use File::Copy;

### CONFIG ################################################################################################################

$version = "0.20";

### FUNCTIONS #############################################################################################################

sub loadconfig {
  print "* Loading config ... ";
  if (!open(CONFIG, $_[0])) { print "FAILED\nERROR: Cannot open config file.\n"; exit; }
  @file = <CONFIG>;
  close(CONFIG);
  foreach $line (@file) {
    chomp($line);
    if ((length($line) != 0) && (substr($line, 0, 1) ne "#")) {
      ($a, $b) = split(/ = /, $line);
      $config{$a} = $b;
    }
  }
  print "done\n";
  return(0);
}

sub loadlanguage {
  print "* Loading language ($config{Language}) ... ";
  if (!open(LANGUAGE, "language/$config{Language}")) { print "FAILED\nERROR: Cannot open language file.\n"; exit; }
  @file = <LANGUAGE>;
  close(LANGUAGE);
  foreach $line (@file) {
    chomp($line);
    if ((length($line) != 0) && (substr($line, 0, 1) ne "#")) {
      if (index($line, ':') != -1) {
        $a = substr($line, 0, index($line, ':'));
        $b = substr($line, index($line, ':')+1, length($line)-index($line, ':')-1);
        $lang[$a] = $b;
      }
    }
  }
  @months = split(',', $lang[11]);
  print "done\n";
  return(0);
}

sub loadstyle {
  print "* Loading style ($config{Style}) ... ";
  if (!open(STYLE, "style/$config{Style}/style.conf")) { print "FAILED\nERROR: Cannot open style file.\n"; exit; }
  @file = <STYLE>;
  close(STYLE);
  foreach $line (@file) {
    chomp($line);
    if ((length($line) != 0) && (substr($line, 0, 1) ne "#")) {
      ($a, $b) = split(/ = /, $line);
      $style{$a} = $b;
    }
  }
  print "done\n";
  return(0);
}

sub splitdate {
  @tmp = ();
  $seperator = substr($config{DateFormat}, 1, 1);
  $pos = index($config{DateFormat}, 'D');
  if ($pos == 0) { $tmp[0] = substr($_[0], 0, index($_[0], $seperator)); } 
  elsif ($pos == 2) { $tmp[0] = substr($_[0], index($_[0], $seperator)+1, rindex($_[0], $seperator)-index($_[0], $seperator)-1); }
  elsif ($pos == 4) { $tmp[0] = substr($_[0], rindex($_[0], $seperator)+1, length($_[0])-rindex($_[0], $seperator)-1); }
  $pos = index($config{DateFormat}, 'M');
  if ($pos == 0) { $tmp[1] = substr($_[0], 0, index($_[0], $seperator)); } 
  elsif ($pos == 2) { $tmp[1] = substr($_[0], index($_[0], $seperator)+1, rindex($_[0], $seperator)-index($_[0], $seperator)-1); }
  elsif ($pos == 4) { $tmp[1] = substr($_[0], rindex($_[0], $seperator)+1, length($_[0])-rindex($_[0], $seperator)-1); }
  $pos = index($config{DateFormat}, 'Y');
  if ($pos == 0) { $tmp[2] = substr($_[0], 0, index($_[0], $seperator)); } 
  elsif ($pos == 2) { $tmp[2] = substr($_[0], index($_[0], $seperator)+1, rindex($_[0], $seperator)-index($_[0], $seperator)-1); }
  elsif ($pos == 4) { $tmp[2] = substr($_[0], rindex($_[0], $seperator)+1, length($_[0])-rindex($_[0], $seperator)-1); }
  return(@tmp);
}

sub cmpdate {
  # 0 -> date1 is erlier than date2
  # 1 -> date1 is equal as date2
  # 2 -> date1 is later than date2
  ($d1, $m1, $y1) = &splitdate($_[0]);
  ($d2, $m2, $y2) = &splitdate($_[1]);
  if ($y1 < $y2) { return(0); } else { if ($y1 > $y2) { return(2); } else {
    if ($m1 < $m2) { return(0); } else { if ($m1 > $m2) { return(2); } else {
      if ($d1 < $d2) { return(0); } else { if ($d1 > $d2) { return(2); } else { return(1); } } } }
    }
  }
  return(0);
}

sub getdate {
  $seperator = substr($config{DateFormat}, 1, 1);
  $a = index($_[0], '-');
  $b = $a;
  $b = index($_[0], $seperator, $b+1);
  $b = index($_[0], $seperator, $b+1);
  $b = index($_[0], '-', $b+1);
  if ($b == -1) { $b = rindex($_[0], '.'); }
  $tmp = substr($_[0], $a+1, $b-$a-1);
  return($tmp);
}

sub getfilename {
  $a = rindex($_[0], '/');
  $tmp = substr($_[0], $a+1, length($_[0])-$a-1);
  return($tmp);
}

sub cmpext {
  # extended datecheck
  # 0 -> date1 is erlier than date2
  # 1 -> date1 is equal as date2
  # 2 -> date1 is later than date2
  $tmp = &getdate($_[0]);
  $date1 = substr($_[0], index($_[0], $tmp)+length($tmp)+1, rindex($_[0], '.')-index($_[0], $tmp)-length($tmp)-1);
  $tmp = &getdate($_[1]);
  $date2 = substr($_[1], index($_[1], $tmp)+length($tmp)+1, rindex($_[1], '.')-index($_[1], $tmp)-length($tmp)-1);
  if ($date1 eq 'tx') { $date1 = 0; } 
  if ($date2 eq 'tx') { $date2 = 0; } 
  if ($date1 == $date2) { return(1); }
  if ($date1 < $date2) { return(0); }
  if ($date1 > $date2) { return(2); }
  return(0);
}

sub readfastmode {
  @oldlogs = ();
  @newlogs = ();
  %fastmode = ();

  open(FAST, substr($config, 0, rindex($config, '.')) . '.fastmode');
  while (<FAST>) {
    chomp($_);
    if ((length($_) != 0) && (substr($_, 0, 1) ne "#")) {
      ($a, $b) = split(/ = /, $_);
      if ($a eq 'done') { push(@oldlogs, $b); }
        else {
          if ($a eq 'Month') {
            ($month, $count) = split(/:/, $b);
            $playmonths[$month] = $count;
          } else {
            if ($a eq 'Hour') {
              ($hour, $count) = split(/:/, $b);
              $playhours[$hour] = $count;
            } else {
              if ($a eq 'Player') {
                ($player, $id, $skill, $kills, $deaths, $suicides, $killstreak, $deathstreak, @weaps) = split("\":\"", $b);
                $players->{$player}->{id} = $id;
                $players->{$player}->{skill} = $skill;
                $players->{$player}->{kills} = $kills;
                $players->{$player}->{deaths} = $deaths;
                $players->{$player}->{suicides} = $suicides;
                $players->{$player}->{killstreak}->{max} = $killstreak;
                $players->{$player}->{killstreak}->{last} = 0;
                $players->{$player}->{deathstreak}->{max} = $deathstreak;
                $players->{$player}->{deathstreak}->{last} = 0;
		$players->{$player}->{captures} = 0;
		$players->{$player}->{returnes} = 0;
                for ($i=0;$i<=$#weapons;$i++) {
                  if ($weaps[$i] != 0) { $players->{$player}->{killweapon}->{$weapons[$i]} = $weaps[$i]; }
                }
                for ($i=($#weapons+1);$i<=($#weapons*2+1);$i++) {
                  if ($weaps[$i] != 0) { $players->{$player}->{deathweapon}->{$weapons[$i-$#weapons-1]} = $weaps[$i]; }
                }
              } else { 
                if ($a eq 'Weapon') {
                  ($weapon, $id, $count) = split("\":\"", $b);
                  $weapons->{$weapon}->{id} = $id;
                  $weapons->{$weapon}->{kills} = $count;
                } else { $fastmode{$a} = $b; }
            }
          }
        }
      }
    }
  }
  close(FAST);

  foreach $log (@logs) {
    $found = 0;
    foreach $oldlog (@oldlogs) {
      if ($log eq $oldlog) { $found = 1; last; }
    }
    if ($found == 0) { push(@newlogs, $log); }
  }

  $begindate = $fastmode{BeginDate};
  $enddate = $fastmode{EndDate};
  $TotalKills = $fastmode{TotalKills};
  $TotalSuicides = $fastmode{TotalSuicides};
  return(0);
}

sub writefastmode {
  push(@logs, @oldlogs);
  if (open(FAST, '>' . substr($config, 0, rindex($config, '.')) . '.fastmode')) {
    print FAST "######################################################\n";
    print FAST "# Soldat Server Statistics Generator - FastMode File #\n";
    print FAST "# For SSSG Version $version" . ' ' x (34-length($version)) . "#\n";
    print FAST "# (c) 2003 CREEQ                                     #\n";
    print FAST "######################################################\n\n";
    print FAST "# DO NOT MODIFY THIS FILE !!!\n\n";
    foreach $log (@logs) {
      print FAST "done = $log\n";
    }
    print FAST "\n";
    print FAST "BeginDate = $begindate\n";
    print FAST "EndDate = $enddate\n";
    print FAST "TotalKills = $TotalKills\n";
    print FAST "TotalSuicides = $TotalSuicides\n\n";
    for ($i=1;$i<=12;$i++) {
      if (defined $playmonths[$i]) { print FAST "Month = $i:$playmonths[$i]\n"; }
    }
    print FAST "\n";
    for ($i=0;$i<24;$i++) {
      if (defined $playhours[$i]) { print FAST "Hour = $i:$playhours[$i]\n"; }
    }
    print FAST "\n";
    foreach $player (keys %{$players}) {
      print FAST "Player = $player\":\"" . $players->{$player}->{id} . "\":\"";
      print FAST $players->{$player}->{skill} . "\":\"";
      print FAST $players->{$player}->{kills} . "\":\"";
      print FAST $players->{$player}->{deaths} . "\":\"";
      print FAST $players->{$player}->{suicides} . "\":\"";
      print FAST $players->{$player}->{killstreak}->{max} . "\":\"";
      print FAST $players->{$player}->{deathstreak}->{max} . "\":\"";
      for ($i=0;$i<=$#weapons;$i++) {
        if (defined $players->{$player}->{killweapon}->{$weapons[$i]}) { print FAST $players->{$player}->{killweapon}->{$weapons[$i]}; } else { print FAST '0'; }
        print FAST "\":\"";
      }
      for ($i=0;$i<=$#weapons;$i++) {
        if (defined $players->{$player}->{deathweapon}->{$weapons[$i]}) { print FAST $players->{$player}->{deathweapon}->{$weapons[$i]}; } else { print FAST '0'; }
        print FAST "\":\"";
      }
      print FAST "\n";
    }
    print FAST "\n";
    foreach $weapon (keys %{$weapons}) {
      print FAST "Weapon = $weapon\":\"" . $weapons->{$weapon}->{id} . "\":\"";
      print FAST $weapons->{$weapon}->{kills} . "\n";
    }
    close(FAST);
  } else { print "FAILED\nWARNING: Cannot write fastmode file.\n"; }
  return(0);
}

sub generatestat {
  @logs = `find $config{LogDir} -name ?ill?og*.txt 2> /dev/null`;
  if ($#logs == -1) { print "FAILED\nERROR: No log files found.\n"; exit; }
  for ($i=0;$i<=$#logs;$i++) { chomp($logs[$i]); }
  $consolelog = `find $config{LogDir} -name ?onsole?og.txt`;
  chomp($consolelog);
  if ($consolelog ne '') {
    open(LOG, $consolelog);
    @consolelog = <LOG>;
    close(LOG);
  } else { print "WARNING: ConsoleLog not found.\n"; }

  if ($config{FastMode} eq 'yes') {
    foreach $log (@logs) {
      if (defined $lastdate) {
        if (&cmpdate(&getdate(&getfilename($lastdate)), &getdate(&getfilename($log))) == 0) { $lastdate = $log; }
        if (&cmpdate(&getdate(&getfilename($lastdate)), &getdate(&getfilename($log))) == 1) {
          if (&cmpext(&getfilename($lastdate), &getfilename($log)) == 0) { $lastdate = $log; }
        }
      }  else { $lastdate = $log; }
    }
    for ($i=0;$i<=$#logs;$i++) {
      if ($logs[$i] eq $lastdate) { splice(@logs, $i, 1); last; }
    }
  }

  if (defined $config{IgnoreUsers}) {
    foreach $ignore (split(/;/, $config{IgnoreUsers})) { $ignores{lc $ignore} = 0; }
    $ignores{''} = 0;
  }

  if (($config{FastMode} eq 'yes') && (-e substr($config, 0, index($config, '.')) . '.fastmode')) {
    print "* Reading fastmode file ... ";
    &readfastmode;
    $lastplayerid = (keys %{$players}) + 2;
    $lastweaponid = (keys %{$weapons}) + 2;
    print "done\n";
  } else {
    @newlogs = @logs;
    @killlog = ();
    $TotalSuicides = 0;
    $TotalKills = 0;
    $lastplayerid = 1;
    $lastweaponid = 1;
  }

  foreach $log (@newlogs) {
    open(LOG, $log);
    push(@killlog, <LOG>);
    close(LOG);
  }
  if ($#killlog != -1) {
    print "* Analyzing Kill-Logs ... ";
    &makestats;
    print "done\n";
    if ($config{FastMode} eq 'yes') {
      print "* Writing fastmode file ... ";
      &writefastmode;
      print "done\n";
    }
  }

  if ($config{FastMode} eq 'yes') {
    @killlog = ();
    open(LOG, $lastdate);
    push(@killlog, <LOG>);
    close(LOG);
    print "* Analyzing NEW Kill-Logs ... ";
    &makestats;
    print "done\n";
  }
  
  if ($consolelog ne '') {
    print "* Analyzing Console-Log ... ";
    &makeconsolestats;
    print "done\n";
  }

  $TotalPlayers = keys %{$players};

  ### RANK ###
  @order = sort { $players->{$b}->{skill} <=> $players->{$a}->{skill} } keys %{$players};
  for ($i=0;$i<=$#order;$i++) {
    $players->{$order[$i]}->{rank} = ($i+1);
  }

  ### WEAPONS ###
  foreach $weapon (keys %{$weapons}) {
    for ($i=0;$i<=$#weapons;$i++) {
      if ($weapons[$i] eq $weapon) { $weapons->{$weapon}->{id} = ($i+1); last; }
    }
  }

  ### AWARDS & MISC CALCULATION ###
  &initawards;
  foreach $player (keys %{$players}) {
    ### MISC CALCULATION ###
    if (defined $ignores{lc $player}) {
      delete($players->{$player});
      next;
    }
    $players->{$player}->{skill} = sprintf("%d", $players->{$player}->{skill});
    if ($players->{$player}->{deaths} != 0) { $players->{$player}->{killdeath} = sprintf("%0.2f", ($players->{$player}->{kills}/$players->{$player}->{deaths})); }
      else { $players->{$player}->{killdeath} = sprintf("%0.2f", $players->{$player}->{kills}); }
    if ($players->{$player}->{deathstreak}->{last} > $players->{$player}->{deathstreak}->{max}) { $players->{$player}->{deathstreak}->{max} = $players->{$player}->{deathstreak}->{last}; }
    if ($players->{$player}->{killstreak}->{last} > $players->{$player}->{killstreak}->{max}) { $players->{$player}->{killstreak}->{max} = $players->{$player}->{killstreak}->{last}; }
    ### AWARDS ###
    if ((defined $awards->{killdeath}) && ($players->{$player}->{killdeath} > $awards->{killdeath}->{value})) {
      $awards->{killdeath}->{value} = $players->{$player}->{killdeath};
      $awards->{killdeath}->{winner} = $player;
    }
    if ((defined $awards->{flagcapture}) && ($players->{$player}->{captures} > $awards->{flagcapture}->{value})) {
      $awards->{flagcapture}->{value} = $players->{$player}->{captures};
      $awards->{flagcapture}->{winner} = $player;
    }
    if ((defined $awards->{flagreturn}) && ($players->{$player}->{returnes} > $awards->{flagreturn}->{value})) {
      $awards->{flagreturn}->{value} = $players->{$player}->{returnes};
      $awards->{flagreturn}->{winner} = $player;
    }
    if ((defined $awards->{killstreak}) && ($players->{$player}->{killstreak}->{max} > $awards->{killstreak}->{value})) {
      $awards->{killstreak}->{value} = $players->{$player}->{killstreak}->{max};
      $awards->{killstreak}->{winner} = $player;
    }
  }
  foreach $award (keys %{$awards}) {
    if ($awards->{$award}->{value} > 0) {
      $players->{$awards->{$award}->{winner}}->{skill} += $awards->{$award}->{skill};
      $players->{$awards->{$award}->{winner}}->{awards}->{$award} = 0;
    }
  }
  return(0);
}

sub initawards {
  $awards->{killdeath}->{value} = 0;
  $awards->{killdeath}->{language} = 42;
  $awards->{killdeath}->{skill} = 10;
  $awards->{flagcapture}->{value} = 0;
  $awards->{flagcapture}->{language} = 43;
  $awards->{flagcapture}->{skill} = 10;
  $awards->{flagreturn}->{value} = 0;
  $awards->{flagreturn}->{language} = 44;
  $awards->{flagreturn}->{skill} = 10;
  $awards->{killstreak}->{value} = 0;
  $awards->{killstreak}->{language} = 45;
  $awards->{killstreak}->{skill} = 10;
  return(0);
}

sub initplayer {
  $players->{$_[0]}->{kills} = 0;
  $players->{$_[0]}->{deaths} = 0;
  $players->{$_[0]}->{suicides} = 0;
  $players->{$_[0]}->{killstreak}->{max} = 0;
  $players->{$_[0]}->{killstreak}->{last} = 0;
  $players->{$_[0]}->{deathstreak}->{max} = 0;
  $players->{$_[0]}->{deathstreak}->{last} = 0;
  $players->{$_[0]}->{id} = $lastplayerid++;
  $players->{$_[0]}->{skill} = 1000;
  $players->{$_[0]}->{captures} = 0;
  $players->{$_[0]}->{returnes} = 0;
  return(0);
}

sub initweapon {
  $weapons->{$_[0]}->{kills} = 0;
  $weapons->{$_[0]}->{id} = $lastweaponid++;
  return(0);
}

sub initmap {
  $maps->{$_[0]}->{played} = 0;
  return(0);
}

sub makeconsolestats {
  foreach $line (@consolelog) {
    while ((substr($line, length($line)-1, 1) eq "\n") || (substr($line, length($line)-1, 1) eq "\r")) { chop($line); }
    if (index($line, 'scores for') != -1) {
      $player = substr($line, 0, index($line, 'scores for')-1);
      if (!defined $players->{$player}) { &initplayer($player); }
      $players->{$player}->{captures} += 1;
      $players->{$player}->{skill} += 5;
    }    
    if (index($line, 'returned the') != -1) {
      $player = substr($line, 0, index($line, 'returned the')-1);
      if (!defined $players->{$player}) { &initplayer($player); }
      $players->{$player}->{returnes} += 1;
      $players->{$player}->{skill} += 5;
    }    
    if (index($line, 'Next map:') != -1) {
      $map = substr($line, rindex($line, ':')+2, length($line)-index($line, 'Next map:')-1);
      if (!defined $maps->{$map}) { &initmap($map); }
      $maps->{$map}->{played} += 1;
    }    
    if ((substr($line, 0, 1) eq '[') && (index($line, '] ') != -1)) {
      $player = substr($line, 1, rindex($line, '] ')-1);
      if ((index($player, 'SERVER MESSAGE') == -1) && (defined $players->{$player})) {
        $message = substr($line, rindex($line, '] ')+2, length($line)-rindex($line, '] ')-2);
        push(@{$players->{$player}->{chat}}, $message);
      }
    }
  }
  return(0);
}

sub makestats {
  foreach $line (@killlog) {
    while ((substr($line, length($line)-1, 1) eq "\n") || (substr($line, length($line)-1, 1) eq "\r")) { chop($line); }
    if ((substr($line, 0, 1) ne " ") && (length($line) > 0)) {
      $date = substr($line, 0, index($line, ' '));
      if (index($date, '/') != -1) {
        $seperator = substr($config{DateFormat}, 1, 1);
        while(index($date, '/') != -1) { substr($date, index($date, '/'), 1) = $seperator; }
      }
      $time = substr($line, length($date)+1, index($line, ' ', length($date)+1)-length($date)-1);
      $killer = substr($line, length($date.$time)+2, index($line, ' / ')-length($date.$time)-2);
      $hour = substr($time, 0, index($time, ':'));
      if (((substr($killer, 0, 4) eq 'p.m.') || (substr($killer, 0, 2) eq 'PM')) && ($hour < 12)) { $hour += 12; }
      if (((substr($killer, 0, 4) eq 'a.m.') || (substr($killer, 0, 2) eq 'AM')) && ($hour == 12)) { $hour = 0; }
      if ((substr($killer, 0, 4) eq 'a.m.') || (substr($killer, 0, 4) eq 'p.m.')) { substr($killer, 0, 5) = ''; }
      if ((substr($killer, 0, 2) eq 'AM') || (substr($killer, 0, 2) eq 'PM')) { substr($killer, 0, 3) = ''; }
      $weapon = substr($line, rindex($line, ' / ')+3, length($line)-rindex($line, ' / '));
      if ($weapon ne 'himself') {
        $victim = substr($line, index($line, ' / ',)+3, rindex($line, ' / ')-index($line, ' / ')-3);
        if (!defined $players->{$victim}) { &initplayer($victim); }
      }
      if (!defined $players->{$killer}) { &initplayer($killer); }

      if (defined $begindate) { if (&cmpdate($begindate, $date) == 2) { $begindate = $date; } } else { $begindate = $date; }
      if (defined $enddate) { if (&cmpdate($enddate, $date) == 0) { $enddate = $date; } } else { $enddate = $date; }
      ($day, $month, $year) = &splitdate($date);
      if (defined $playhours[$hour]) { $playhours[$hour] += 1; } else { $playhours[$hour] = 1; }
      if (defined $playmonths[$month]) { $playmonths[$month] += 1; } else { $playmonths[$month] = 1; }
      if ($weapon eq 'himself') { $TotalSuicides += 1; }
      $TotalKills += 1;
      if (($weapon ne 'himself') && ($weapon ne '')) {
        if (!defined $weapons->{$weapon}) { &initweapon($weapon); }
        $players->{$killer}->{kills} += 1;
        $players->{$killer}->{killstreak}->{last} += 1;
        $players->{$victim}->{deaths} += 1;
        $players->{$victim}->{deathstreak}->{last} += 1;
        if ($players->{$killer}->{deathstreak}->{last} > $players->{$killer}->{deathstreak}->{max}) { $players->{$killer}->{deathstreak}->{max} = $players->{$killer}->{deathstreak}->{last}; }
        if ($players->{$victim}->{killstreak}->{last} > $players->{$victim}->{killstreak}->{max}) { $players->{$victim}->{killstreak}->{max} = $players->{$victim}->{killstreak}->{last}; }
        $players->{$killer}->{deathstreak}->{last} = 0;
        $players->{$victim}->{killstreak}->{last} = 0;
        $skill = ($players->{$victim}->{skill} / $players->{$killer}->{skill}) * 10;
	if ($skill > 100) { $skill = 100; }
        $players->{$killer}->{skill} += $skill;
       	if (($players->{$victim}->{skill} < 2000) && (($players->{$victim}->{skill}-10) > 0)) { $players->{$victim}->{skill} -= 10; }
          else { $players->{$victim}->{skill} -= $skill; }
        $weapons->{$weapon}->{kills} += 1;
        if (defined $players->{$killer}->{killweapon}->{$weapon}) { $players->{$killer}->{killweapon}->{$weapon} += 1; } else { $players->{$killer}->{killweapon}->{$weapon} = 1; }
        if (defined $players->{$victim}->{deathweapon}->{$weapon}) { $players->{$victim}->{deathweapon}->{$weapon} += 1; } else { $players->{$victim}->{deathweapon}->{$weapon} = 1; }
      } else {
        $players->{$killer}->{deaths} += 1;
        $players->{$killer}->{suicides} += 1;
   	if (($players->{$killer}->{skill}-3) > 0) { $players->{$killer}->{skill} -= 3; }
      }
    }
  }
  return(0);
}

sub playerstring {
  $tmp = "<a href=\"$_[1]player" . $players->{$_[0]}->{id} . "/\">$_[0]</a>";
  if ($players->{$_[0]}->{skill} >= 1300) {
    $tmp = $tmp . " " . ("<img src=\"$_[1]star.gif\" height=\"10\" alt=\"Skill >= " . (1000+(300*int(($players->{$_[0]}->{skill}-1000)/300))) . "\">") x int(($players->{$_[0]}->{skill}-1000)/300);
  }
  foreach $tmp2 (keys %{$players->{$_[0]}->{awards}}) {
    $tmp = $tmp . " <img src=\"$_[1]award.gif\" height=\"10\" alt=\"Award: " . $lang[$awards->{$tmp2}->{language}]  . "\">";
  }
  return($tmp);
}

sub resetvars {
  %config = ();
  @lang = ();
  %style = ();
  @logs = ();
  @newlogs = ();
  @oldlogs = ();
  @killlog = ();
  @consolelog = ();
  undef $lastdate;
  @playmonths = ();
  @playhours = ();
  %{$players} = ();
  %{$maps} = ();
  %{$weapons} = ();
  %{$awards} = ();
  %ignores = ();
  %fastmode = ();
  %playerdetails = ();
  %weapondetails = ();
  return(0);
}

sub createHTML {
  if (!open(INDEX, ">$config{StatDir}index.html")) { print "FAILED\nERROR: Cannot write stat file.\n"; exit; }

  ### HEAD ###
  print INDEX "<html>\n";
  print INDEX "<head>\n";
  print INDEX "<title>$config{ServerName} - Soldat Server $lang[0]</title>\n";
  print INDEX "</head>\n";
  print INDEX "<body bgcolor=\"$style{BGColor}\" text=\"$style{TextColor}\" link=\"$style{LinkColor}\" vlink=\"$style{LinkColor}\" alink=\"$style{LinkColor}\"";
  if ($style{BackGround} ne '') { print INDEX " background=\"$style{BackGround}\""; }
  print INDEX ">\n";
  print INDEX "<center>\n";
  print INDEX "<font face=\"$style{FontFace}\" size=\"5\"><b><u>$config{ServerName}</u></b></font><br>\n";
  print INDEX "<font face=\"$style{FontFace}\" size=\"2\"><b>$lang[1] <a href=\"mailto:$config{YourMail}\">$config{YourName}</a></b></font><br><br>\n";
  print INDEX "<font face=\"$style{FontFace}\" size=\"2\">$lang[2] " . localtime($begin) . ".</font><br><br>\n";

  ## GENERAL INFO ###
  if ($config{ShowGeneralInfo} eq 'yes') {
    print INDEX "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print INDEX "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[3]</b></font></td></tr>\n";
    print INDEX "</table>\n";
    print INDEX "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"33%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[4]</b></font></td>\n";
    print INDEX "<td width=\"67%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$config{ServerName}</b></font></td>\n";
    print INDEX "</tr>\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"33%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[5]</b></font></td>\n";
    print INDEX "<td width=\"67%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$config{ServerIP} @ $config{ServerPort}</b></font></td>\n";
    print INDEX "</tr>\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"33%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[6]</b></font></td>\n";
    print INDEX "<td width=\"67%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$begindate - $enddate</b></font></td>\n";
    print INDEX "</tr>\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"33%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[7]</b></font></td>\n";
    print INDEX "<td width=\"67%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$TotalPlayers</b></font></td>\n";
    print INDEX "</tr>\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"33%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[8]</b></font></td>\n";
    print INDEX "<td width=\"67%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$TotalKills</b></font></td>\n";
    print INDEX "</tr>\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"33%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[9]</b></font></td>\n";
    print INDEX "<td width=\"67%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$TotalSuicides</b></font></td>\n";
    print INDEX "</tr>\n";
    print INDEX "</table>\n";
    print INDEX "<br><br>\n";
  }

  ### KILLS BY MONTH ###
  if ($config{ShowKillsByMonth} eq 'yes') {
    print INDEX "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print INDEX "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[10]</b></font></td></tr>\n";
    print INDEX "</table>\n";
    print INDEX "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print INDEX "<tr>\n";
    $maxmonth = 0;
    for ($month=1;$month<=12;$month++) {
      if (!defined $playmonths[$month]) { $playmonths[$month] = 0; }
      if ($playmonths[$month] > $maxmonth) { $maxmonth = $playmonths[$month]; }
    }
    for ($month=1;$month<=12;$month++) {
      print INDEX "<td width=\"8%\" bgcolor=\"$style{TableColor2}\" align=\"center\" valign=\"bottom\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"1\"><br>" . sprintf("%0.0f", $playmonths[$month]/$TotalKills*100) . "%<br><img src=\"vbar.jpg\" width=\"15\" height=\"" . ($playmonths[$month]/$maxmonth*100)  . "\"></b></font></td>\n";
    }
    print INDEX "</tr><tr>\n";
    for ($month=1;$month<=12;$month++) {
      print INDEX "<td width=\"8%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"2\"><b>$months[$month-1]</b></font></td>\n";
    }
    print INDEX "</tr>\n";
    print INDEX "</table>\n";
    print INDEX "<br><br>\n";
  }

  ### KILLS BY HOUR ###
  if ($config{ShowKillsByHour} eq 'yes') {
    print INDEX "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print INDEX "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[12]</b></font></td></tr>\n";
    print INDEX "</table>\n";
    print INDEX "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print INDEX "<tr>\n";
    $maxhour = 0;
    for ($hour=0;$hour<24;$hour++) {
      if (!defined $playhours[$hour]) { $playhours[$hour] = 0; }
      if ($playhours[$hour] > $maxhour) { $maxhour = $playhours[$hour]; }
    }
    for ($hour=0;$hour<24;$hour++) {
      print INDEX "<td width=\"4%\" bgcolor=\"$style{TableColor2}\" align=\"center\" valign=\"bottom\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"1\"><br>" . sprintf("%0.0f", $playhours[$hour]/$TotalKills*100) . "%<br><img src=\"vbar.jpg\" width=\"15\" height=\"" . ($playhours[$hour]/$maxhour*100)  . "\"></b></font></td>\n";
    }
    print INDEX "</tr><tr>\n";
    for ($hour=0;$hour<24;$hour++) {
      print INDEX "<td width=\"4%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"2\"><b>$hour</b></font></td>\n";
    }
    print INDEX "</tr>\n";
    print INDEX "</table>\n";
    print INDEX "<br><br>\n";
  }

  ### TOP KILLERS ###
  if ($config{ShowTopKillers} eq 'yes') {
    print INDEX "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print INDEX "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[13]</b></font></td></tr>\n";
    print INDEX "</table>\n";
    print INDEX "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"35%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[14]</b></font></td>\n";
    print INDEX "<td width=\"10%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[29]</b></font></td>\n";
    print INDEX "<td width=\"10%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[15]</b></font></td>\n";
    print INDEX "<td width=\"10%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[16]</b></font></td>\n";
    print INDEX "<td width=\"35%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[17]</b></font></td>\n";
    print INDEX "</tr>\n";
    $count = 0;
    foreach $player (sort {$players->{$a}->{rank} <=> $players->{$b}->{rank}} keys %{$players}) {
      if ($players->{$player}->{kills} == 0) { last; }
      $playerdetails{$player} = 0;
      $max = 0;
      foreach (keys %{$players->{$player}->{killweapon}}) {
        if ($players->{$player}->{killweapon}->{$_} > $max) { $max = $players->{$player}->{killweapon}->{$_}; $favweapon = $_; }
      }
      $weapondetails{$favweapon} = 0;
      print INDEX "<tr>\n";
      print INDEX "<td width=\"35%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . &playerstring($player, '') . "</b></font></td>\n";
      print INDEX "<td width=\"10%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{skill} . "</b></font></td>\n";
      print INDEX "<td width=\"10%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$players->{$player}->{kills}</b></font></td>\n";
      print INDEX "<td width=\"10%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$players->{$player}->{killdeath}</b></font></td>\n";
      print INDEX "<td width=\"35%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$favweapon</b></font></td>\n";
      print INDEX "</tr>\n";
      if (++$count == 10) { last; }
    }
    print INDEX "</table>\n";
    print INDEX "<br><br>\n";
  }

  ### TOP VICTIMS ###
  if ($config{ShowTopVictims} eq 'yes') {
    print INDEX "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print INDEX "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[18]</b></font></td></tr>\n";
    print INDEX "</table>\n";
    print INDEX "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"75%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[14]</b></font></td>\n";
    print INDEX "<td width=\"25%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[19]</b></font></td>\n";
    print INDEX "</tr>\n";
    $count = 0;
    foreach $player (sort {$players->{$b}->{deaths} <=> $players->{$a}->{deaths}} keys %{$players}) {
      if ($players->{$player}->{deaths} == 0) { last; }
      $playerdetails{$player} = 0;
      print INDEX "<tr>\n";
      print INDEX "<td width=\"75%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . &playerstring($player, '') . "</b></font></td>\n";
      print INDEX "<td width=\"25%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$players->{$player}->{deaths}</b></font></td>\n";
      print INDEX "</tr>\n";
      if (++$count == 10) { last; }
    }
    print INDEX "</table>\n";
    print INDEX "<br><br>\n";
  }

  ### AWARDS ###
  if ($config{ShowAwards} eq 'yes') {
    print INDEX "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print INDEX "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[40]</b></font></td></tr>\n";
    print INDEX "</table>\n";
    print INDEX "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"70%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[41]</b></font></td>\n";
    print INDEX "<td width=\"30%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[14]</b></font></td>\n";
    print INDEX "</tr>\n";
    foreach $award (keys %{$awards}) {
      if ($awards->{$award}->{value} == 0) { next; }
      $playerdetails{$awards->{$award}->{winner}} = 0;
      print INDEX "<tr>\n";
      print INDEX "<td width=\"70%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[$awards->{$award}->{language}] ($awards->{$award}->{value})</b></font></td>\n";
      print INDEX "<td width=\"30%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . &playerstring($awards->{$award}->{winner}, '') . "</b></font></td>\n";
      print INDEX "</tr>\n";
    }
    print INDEX "</table>\n";
    print INDEX "<br><br>\n";
  }

  ### USED WEAPONS ###
  if ($config{ShowUsedWeapons} eq 'yes') {
    print INDEX "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print INDEX "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[20]</b></font></td></tr>\n";
    print INDEX "</table>\n";
    print INDEX "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"75%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[21]</b></font></td>\n";
    print INDEX "<td width=\"25%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[15]</b></font></td>\n";
    print INDEX "</tr>\n";
    foreach $weapon (sort {$weapons->{$b}->{kills} <=> $weapons->{$a}->{kills}} keys %{$weapons}) {
      $weapondetails{$weapon} = 0;
      print INDEX "<tr>\n";
      print INDEX "<td width=\"75%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b><a href=\"weapon" . $weapons->{$weapon}->{id} . "\">$weapon</a></b></font></td>\n";
      print INDEX "<td width=\"25%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $weapons->{$weapon}->{kills} . "</b></font></td>\n";
      print INDEX "</tr>\n";
    }
    print INDEX "</table>\n";
    print INDEX "<br><br>\n";
  }

  ### CAPTURE THE FLAG ###
  if ($config{ShowCaptureTheFlag} eq 'yes') {
    print INDEX "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print INDEX "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[31]</b></font></td></tr>\n";
    print INDEX "</table>\n";
    print INDEX "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[14]</b></font></td>\n";
    print INDEX "<td width=\"25%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[32]</b></font></td>\n";
    print INDEX "<td width=\"25%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[33]</b></font></td>\n";
    print INDEX "</tr>\n";
    $count = 0;
    foreach $player (sort {$players->{$b}->{captures} <=> $players->{$a}->{captures}} keys %{$players}) {
      if ($players->{$player}->{captures} == 0) { last; }
      $playerdetails{$player} = 0;
      print INDEX "<tr>\n";
      print INDEX "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . &playerstring($player, '') . "</b></font></td>\n";
      print INDEX "<td width=\"25%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{captures} . "</b></font></td>\n";
      print INDEX "<td width=\"25%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{returnes} . "</b></font></td>\n";
      print INDEX "</tr>\n";
      if (++$count == 5) { last; }
    }
    print INDEX "</table>\n";
    print INDEX "<br><br>\n";
  }

  ### PLAYED MAPS ###
  if ($config{ShowPlayedMaps} eq 'yes') {
    print INDEX "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print INDEX "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[34]</b></font></td></tr>\n";
    print INDEX "</table>\n";
    print INDEX "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"75%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[35]</b></font></td>\n";
    print INDEX "<td width=\"25%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[36]</b></font></td>\n";
    print INDEX "</tr>\n";
    $count = 0;
    foreach $map (sort {$maps->{$b}->{played} <=> $maps->{$a}->{played}} keys %{$maps}) {
      if ($maps->{$map}->{played} == 0) { last; }
      print INDEX "<tr>\n";
      print INDEX "<td width=\"75%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$map</b></font></td>\n";
      print INDEX "<td width=\"25%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $maps->{$map}->{played} . "</b></font></td>\n";
      print INDEX "</tr>\n";
      if (++$count == 5) { last; }
    }
    print INDEX "</table>\n";
    print INDEX "<br><br>\n";
  }

  ### TOP CHATTERS ###
  if ($config{ShowTopChatters} eq 'yes') {
    print INDEX "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print INDEX "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[37]</b></font></td></tr>\n";
    print INDEX "</table>\n";
    print INDEX "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print INDEX "<tr>\n";
    print INDEX "<td width=\"35%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[14]</b></font></td>\n";
    print INDEX "<td width=\"15%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[38]</b></font></td>\n";
    print INDEX "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[39]</b></font></td>\n";
    print INDEX "</tr>\n";
    $count = 0;
    foreach $player (sort {$#{$players->{$b}->{chat}} <=> $#{$players->{$a}->{chat}}} keys %{$players}) {
      if ($#{$players->{$player}->{chat}} == -1) { last; }
      $playerdetails{$player} = 0;
      print INDEX "<tr>\n";
      print INDEX "<td width=\"35%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . &playerstring($player, '') . "</b></font></td>\n";
      print INDEX "<td width=\"15%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . ($#{$players->{$player}->{chat}}+1) . "</b></font></td>\n";
      print INDEX "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . ${$players->{$player}->{chat}}[int(rand($#{$players->{$player}->{chat}}))] . "</b></font></td>\n";
      print INDEX "</tr>\n";
      if (++$count == 10) { last; }
    }
    print INDEX "</table>\n";
    print INDEX "<br><br>\n";
  }

  ### GENERATE WEAPON DETAILS ###
  foreach $weapon (keys %weapondetails) {
    mkdir $config{StatDir} . "weapon" . $weapons->{$weapon}->{id}, 0755;
    if (!open(DETAIL, ">" . $config{StatDir} . "weapon" . $weapons->{$weapon}->{id} . "/index.html")) { print "\nERROR: Cannot write stat file ($weapon).\n"; exit; }
    print DETAIL "<html>\n";
    print DETAIL "<head>\n";
    print DETAIL "<title>$config{ServerName} - Soldat Server $lang[0] - $player</title>\n";
    print DETAIL "</head>\n";
    print DETAIL "<body bgcolor=\"$style{BGColor}\" text=\"$style{TextColor}\" link=\"$style{LinkColor}\" vlink=\"$style{LinkColor}\" alink=\"$style{LinkColor}\"";
    if ($style{BackGround} ne '') { print DETAIL " background=\"../$style{BackGround}\""; }
    print DETAIL ">\n";
    print DETAIL "<center>\n";
    print DETAIL "<font face=\"$style{FontFace}\" size=\"5\"><b><u>$config{ServerName}</u></b></font><br>\n";
    print DETAIL "<font face=\"$style{FontFace}\" size=\"2\"><b><a href=\"..\">$lang[26]</a></b></font><br><br>\n";
    print DETAIL "<font face=\"$style{FontFace}\" size=\"3\"><b>$lang[21]: $weapon</b></font><br><br>\n";
  
    ## GENERAL INFO ###
    print DETAIL "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print DETAIL "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[27]</b></font></td></tr>\n";
    print DETAIL "</table>\n";
    print DETAIL "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[8]</b></font></td>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $weapons->{$weapon}->{kills} . "</b></font></td>\n";
    print DETAIL "</tr>\n";
    print DETAIL "</table>\n";
    print DETAIL "<br><br>\n";

    ### TOP KILLERS ###
    print DETAIL "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print DETAIL "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[13]</b></font></td></tr>\n";
    print DETAIL "</table>\n";
    print DETAIL "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"75%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[14]</b></font></td>\n";
    print DETAIL "<td width=\"25%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[15]</b></font></td>\n";
    print DETAIL "</tr>\n";
    $count = 0;
    foreach $player (sort {$players->{$b}->{killweapon}->{$weapon} <=> $players->{$a}->{killweapon}->{$weapon}} keys %{$players}) {
      if ($players->{$player}->{killweapon}->{$weapon} == 0) { last; }
      $playerdetails{$player} = 0;
      print DETAIL "<tr>\n";
      print DETAIL "<td width=\"75%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . &playerstring($player, '../') . "</b></font></td>\n";
      print DETAIL "<td width=\"25%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{killweapon}->{$weapon} . "</b></font></td>\n";
      print DETAIL "</tr>\n";
      if (++$count == 5) { last; }
    }
    print DETAIL "</table>\n";
    print DETAIL "<br><br>\n";

    ### TOP VICTIMS ###
    print DETAIL "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print DETAIL "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[18]</b></font></td></tr>\n";
    print DETAIL "</table>\n";
    print DETAIL "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"75%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[14]</b></font></td>\n";
    print DETAIL "<td width=\"25%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[19]</b></font></td>\n";
    print DETAIL "</tr>\n";
    $count = 0;
    foreach $player (sort {$players->{$b}->{deathweapon}->{$weapon} <=> $players->{$a}->{deathweapon}->{$weapon}} keys %{$players}) {
      if ($players->{$player}->{deathweapon}->{$weapon} == 0) { last; }
      $playerdetails{$player} = 0;
      print DETAIL "<tr>\n";
      print DETAIL "<td width=\"75%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b><a href=\"../player" . $players->{$player}->{id} . "/\">$player</a></b></font></td>\n";
      print DETAIL "<td width=\"25%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{deathweapon}->{$weapon} . "</b></font></td>\n";
      print DETAIL "</tr>\n";
      if (++$count == 5) { last; }
    }
    print DETAIL "</table>\n";
    print DETAIL "<br><br>\n";

    print DETAIL "<font face=\"$style{FontFace}\" size=\"2\"><b><a href=\"..\">$lang[26]</a></b></font><br><br>\n";
    print DETAIL "</center>\n";
    print DETAIL "</body>\n";
    print DETAIL "</html>\n";
    close(DETAIL);
  }

  ### GENERATE PLAYER DETAILS ###
  foreach $player (keys %playerdetails) {
    mkdir $config{StatDir} . "player" . $players->{$player}->{id}, 0755;
    if (!open(DETAIL, ">" . $config{StatDir} . "player" . $players->{$player}->{id} . "/index.html")) { print "\nERROR: Cannot write stat file ($player).\n"; exit; }
    print DETAIL "<html>\n";
    print DETAIL "<head>\n";
    print DETAIL "<title>$config{ServerName} - Soldat Server $lang[0] - $player</title>\n";
    print DETAIL "</head>\n";
    print DETAIL "<body bgcolor=\"$style{BGColor}\" text=\"$style{TextColor}\" link=\"$style{LinkColor}\" vlink=\"$style{LinkColor}\" alink=\"$style{LinkColor}\"";
    if ($style{BackGround} ne '') { print DETAIL " background=\"../$style{BackGround}\""; }
    print DETAIL ">\n";
    print DETAIL "<center>\n";
    print DETAIL "<font face=\"$style{FontFace}\" size=\"5\"><b><u>$config{ServerName}</u></b></font><br>\n";
    print DETAIL "<font face=\"$style{FontFace}\" size=\"2\"><b><a href=\"..\">$lang[26]</a></b></font><br><br>\n";
    print DETAIL "<font face=\"$style{FontFace}\" size=\"3\"><b>$lang[14]: " . &playerstring($player, '../') . "</b></font><br><br>\n";

    ## GENERAL INFO ###
    print DETAIL "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print DETAIL "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[27]</b></font></td></tr>\n";
    print DETAIL "</table>\n";
    print DETAIL "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[30]</b></font></td>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{rank} . "</b></font></td>\n";
    print DETAIL "</tr>\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[29]</b></font></td>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{skill} . "</b></font></td>\n";
    print DETAIL "</tr>\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[8]</b></font></td>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{kills} . "</b></font></td>\n";
    print DETAIL "</tr>\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[28]</b></font></td>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{deaths} . "</b></font></td>\n";
    print DETAIL "</tr>\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[16]</b></font></td>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$players->{$player}->{killdeath}</b></font></td>\n";
    print DETAIL "</tr>\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[9]</b></font></td>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{suicides} . "</b></font></td>\n";
    print DETAIL "</tr>\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[45]</b></font></td>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{killstreak}->{max} . "</b></font></td>\n";
    print DETAIL "</tr>\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[46]</b></font></td>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{deathstreak}->{max} . "</b></font></td>\n";
    print DETAIL "</tr>\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[32]</b></font></td>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{captures} . "</b></font></td>\n";
    print DETAIL "</tr>\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[33]</b></font></td>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{returnes} . "</b></font></td>\n";
    print DETAIL "</tr>\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[38]</b></font></td>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . ($#{$players->{$player}->{chat}}+1) . "</b></font></td>\n";
    print DETAIL "</tr>\n";
    print DETAIL "</table>\n";
    print DETAIL "<br><br>\n";

    ### AWARD ###
    if (defined $players->{$player}->{awards}) {
      print DETAIL "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
      print DETAIL "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[40]</b></font></td></tr>\n";
      print DETAIL "</table>\n";
      print DETAIL "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
      print DETAIL "<tr>\n";
      print DETAIL "<td width=\"70%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[41]</b></font></td>\n";
      print DETAIL "<td width=\"30%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[47]</b></font></td>\n";
      print DETAIL "</tr>\n";
      foreach $award (keys %{$awards}) {
        if (($awards->{$award}->{value} != 0) && $awards->{$award}->{winner} eq $player) {    
          print DETAIL "<tr>\n";
          print DETAIL "<td width=\"70%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[$awards->{$award}->{language}]</b></font></td>\n";
          print DETAIL "<td width=\"30%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$awards->{$award}->{value}</b></font></td>\n";
          print DETAIL "</tr>\n";
        }
      }
      print DETAIL "</table>\n";
      print DETAIL "<br><br>\n";
    }

    ### USED WEAPONS ###
    print DETAIL "<table width=\"90%\" border=\"$style{TableBorder}\" cellspacing=\"0\">\n";
    print DETAIL "<tr><td bgcolor=\"$style{TableTopicColor}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[20]</b></font></td></tr>\n";
    print DETAIL "</table>\n";
    print DETAIL "<table width=\80%\" border=\"$style{TableBorder}\" cellspacing=\"1\">\n";
    print DETAIL "<tr>\n";
    print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[21]</b></font></td>\n";
    print DETAIL "<td width=\"25%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[15]</b></font></td>\n";
    print DETAIL "<td width=\"25%\" bgcolor=\"$style{TableColor1}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>$lang[19]</b></font></td>\n";
    print DETAIL "</tr>\n";
    foreach $weapon (keys %{$players->{$player}->{deathweapon}}) { 
     if (!defined $players->{$player}->{killweapon}->{$weapon}) { $players->{$player}->{killweapon}->{$weapon} = 0; }
    }
    foreach $weapon (sort {$players->{$player}->{killweapon}->{$b} <=> $players->{$player}->{killweapon}->{$a}} keys %{$players->{$player}->{killweapon}}) {
      if (!defined $players->{$player}->{deathweapon}->{$weapon}) { $players->{$player}->{deathweapon}->{$weapon} = 0; }
      print DETAIL "<tr>\n";
      print DETAIL "<td width=\"50%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b><a href=\"../weapon" . $weapons->{$weapon}->{id} . "\">$weapon</a></b></font></td>\n";
      print DETAIL "<td width=\"25%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{killweapon}->{$weapon} . "</b></font></td>\n";
      print DETAIL "<td width=\"25%\" bgcolor=\"$style{TableColor2}\" align=\"center\" style=\"$style{TDStyle}\"><font color=\"$style{TableTopicTextColor}\" size=\"3\"><b>" . $players->{$player}->{deathweapon}->{$weapon} . "</b></font></td>\n";
      print DETAIL "</tr>\n";
    }
    print DETAIL "</table>\n";
    print DETAIL "<br><br>\n";
 
    print DETAIL "<font face=\"$style{FontFace}\" size=\"2\"><b><a href=\"..\">$lang[26]</a></b></font><br><br>\n";
    print DETAIL "</center>\n";
    print DETAIL "</body>\n";
    print DETAIL "</html>\n";
    close(DETAIL);
  }

  ### FOOT ###
  $gentime = time-$begin;
  $hours = int ($gentime / 3600);
  $gentime -= $hours * 3600;
  $minutes = int ($gentime / 60);
  $gentime -= $minutes * 60;
  $seconds = $gentime;
  $gentime = '';
  if ($hours > 0) { $gentime = "$hours $lang[22] "; }
  if ($minutes > 0) { $gentime = $gentime . "$minutes $lang[24] "; }
  $gentime = $gentime . "$seconds $lang[25]";
  print INDEX "<font face=\"$style{FontFace}\" size=\"1\"><b>\n";
  print INDEX "<a href=\"http://fool.homelinux.org/sssg/\" target=\"_blank\">Soldat Server Statistics Generator</a> - Version $version - &copy 2003 <a href=\"mailto:creeq\@web.de?subject=solstat\">CREEQ</a><br>\n";
  print INDEX "$lang[22]: $gentime\n";
  print INDEX "</font>\n";
  print INDEX "</center>\n";
  print INDEX "</body>\n";
  print INDEX "</html>\n";

  close(INDEX);

  return(0);
}

### MAIN ###

$| = 1; # autoflush on
@weapons = ('Barret M82A1', 'Grenade', 'M79', 'Desert Eagles', 'Steyr AUG', 'Ak-74', 'Spas-12', 'XM214 Minigun', 'USSOCOM', 'HK MP5', 'FN Minimi', 'Ruger 77', 'Clusters', 'Hands', 'Bow', 'Flame Bow', 'Flamer', 'Chainsaw', 'Combat Knife', 'LAW', 'Stationary Gun');

print "######################################\n";
print "# Soldat Server Statistics Generator #\n";
print "# Version $version" . " " x (27-length($version)) . "#\n";
print "# (c) 2003 CREEQ                     #\n";
print "######################################\n\n";

if ($#ARGV != -1) { @configs = @ARGV; } else { @configs = <*.conf>; }

foreach $config (@configs) {
  print "*** Processing \"$config\" now ***\n";
  &resetvars;
  $begin = time;
  &loadconfig($config);
  &loadlanguage;
  &loadstyle;
  &generatestat;
  print "* Creating HTML file(s) ... ";
  mkdir $config{StatDir}, 0755;
  system("rm -rf $config{StatDir}*");
  copy("style/$config{Style}/vbar.jpg", $config{StatDir});
  copy("style/$config{Style}/star.gif", $config{StatDir});
  copy("style/$config{Style}/award.gif", $config{StatDir});
  if ($style{BackGround} ne '') { copy("style/$config{Style}/$style{BackGround}", $config{StatDir}); }

  &createHTML;

  print "done\n";
  print "*** Generation Complete ***\n\n";
}
